#include "buildingeditordialog.h"
#include "buildingdelegate.h"
#include "ui_buildingeditordialog.h"

#include <QSqlQuery>

BuildingEditorDialog::BuildingEditorDialog(QWidget *parent) :
        QWidget(parent),
        _ui(new Ui::BuildingEditorDialog)
{
    _ui->setupUi(this);

    setupModel();
    setupFilter();
    setupUi();
}

BuildingEditorDialog::~BuildingEditorDialog()
{
    delete _ui;
    delete _buildingModel;
}

void BuildingEditorDialog::filterButton_Clicked()
{
    if (_ui->filterButton->text() == trUtf8("Szűrés")) // ha a modell nincs szűrve
    {
        _ui->nameLineEdit->setEnabled(false);
        _ui->cityComboBox->setEnabled(false);
        _ui->stateComboBox->setEnabled(false);
        _ui->filterButton->setText(trUtf8("Szűrő törlése"));

        QAbstractItemModel *currentModel = _buildingModel; // léptetjük az aktuális modellt a megszűrtek között

        if (_ui->nameLineEdit->text() != "") // ha van szöveg a szövegdobozban
        {
            _nameSortFilterModel->setSourceModel(currentModel); // forrás tábla
            _nameSortFilterModel->setFilterKeyColumn(1); // szűrés oszlopa
            _nameSortFilterModel->setFilterCaseSensitivity(Qt::CaseInsensitive); // szűrés nem függ a kis/nagybetűtől
            _nameSortFilterModel->setFilterRegExp(_ui->nameLineEdit->text()); // szűrés reguláris kifejezéssel
            currentModel = _nameSortFilterModel;
        }
        if (_ui->cityComboBox->currentText() != "mind")
        {
            _citySortFilterModel->setSourceModel(currentModel); // szűrés a városra
            _citySortFilterModel->setFilterKeyColumn(2);
            _citySortFilterModel->setFilterFixedString(_ui->cityComboBox->currentText()); // szűrés pontos illeszkedésre
            // mivel a DisplayRole-t szűrjük, ezért a legördülő menü szövege kell
            currentModel = _citySortFilterModel;
        }
        if (_ui->stateComboBox->currentText() != "mind")
        {
            _stateSortFilterModel->setSourceModel(currentModel); // szűrés az állapotra
            _stateSortFilterModel->setFilterKeyColumn(4);
            _stateSortFilterModel->setFilterFixedString(QString::number(_ui->stateComboBox->currentIndex() - 1)); // szűrés pontos illeszkedésre
            currentModel = _stateSortFilterModel;
        }

        // az ár szűrést mindenképpen alkalmazzuk
        _priceSortFilterModel->setSourceModel(currentModel);
        _priceSortFilterModel->setFilterPriceBoundaries(_ui->minPriceSpinBox->value(), _ui->maxPriceSpinBox->value());
        currentModel = _priceSortFilterModel;

        _ui->buildingTableView->setModel(currentModel); // szűrések után kialakult modell beállítása
    }
    else
    {
        _ui->nameLineEdit->setEnabled(true);
        _ui->cityComboBox->setEnabled(true);
        _ui->stateComboBox->setEnabled(true);
        _ui->filterButton->setText(trUtf8("Szűrés"));
        _ui->buildingTableView->setModel(_buildingModel); // szűrés nélküli modell visszaállítása
    }
}

void BuildingEditorDialog::buildingTableView_SelectionChanged(const QModelIndex& index)
{
    if (index.isValid())
    {
        _buildingSortFilterModel->setSourceModel(_apartmentModel); // szűrés az épületre
        _buildingSortFilterModel->setFilterKeyColumn(1);
        _buildingSortFilterModel->setFilterFixedString(_ui->buildingTableView->model()->data(_ui->buildingTableView->model()->index(index.row(), 1), Qt::EditRole).toString());
        // a szűréskor a táblamegjelenítőhöz aktuálisan társított modellt kell vennünk
        // a szűrést a szerkesztett tartalomra, vagyis az épület azonosítóra végezzük
        _ui->apartmentTableView->setModel(_buildingSortFilterModel);
    }
    else // szűrő kikapcsolása
    {
        _ui->apartmentTableView->setModel(_apartmentModel);
    }
}

void BuildingEditorDialog::setupModel()
{
    _buildingModel = new BuildingTableModel(this);
    _buildingModel->select();
    _apartmentModel = new ApartmentTableModel(this);
    _apartmentModel->select();

    _buildingSelectionModel = new QItemSelectionModel(_buildingModel); // a szelekciókezelést rállítjuk az épületek modellre

    _nameSortFilterModel = new QSortFilterProxyModel(this); // szűrő és rendező modell
    _citySortFilterModel = new QSortFilterProxyModel(this);
    _stateSortFilterModel = new QSortFilterProxyModel(this);
    _priceSortFilterModel = new PriceSortFilterProxyModel(this);
    _buildingSortFilterModel = new QSortFilterProxyModel(this);
}

void BuildingEditorDialog::setupFilter()
{
    QSqlQuery query;
    query.exec("select name from city");

    _ui->cityComboBox->addItem("mind"); // összes város felvétele
    while (query.next())
    {
        _ui->cityComboBox->addItem(query.value(0).toString()); // egy város felvétele
    }

    _ui->stateComboBox->addItem(trUtf8("mind"));
    _ui->stateComboBox->addItem(trUtf8("foglalható"));
    _ui->stateComboBox->addItem(trUtf8("felújítás alatt"));
    _ui->stateComboBox->addItem(trUtf8("nem foglalható"));
}

void BuildingEditorDialog::setupUi()
{
    // épületek tábla megjelenítője:
    _ui->buildingTableView->setModel(_buildingModel);
    _ui->buildingTableView->resizeColumnsToContents();
    _ui->buildingTableView->setSelectionBehavior(QAbstractItemView::SelectRows); // teljes sor kijelölése
    _ui->buildingTableView->setEditTriggers(QAbstractItemView::NoEditTriggers); // a táblaszerkesztés kikapcsolása a megjelenítőn
    _ui->buildingTableView->setItemDelegate(new BuildingDelegate());
    _ui->buildingTableView->setColumnHidden(0, true);
    // szelekciós modell megadása, a változáskezeléshez:
    _ui->buildingTableView->setSelectionModel(_buildingSelectionModel);

    // apartmanok tábla megjelenítője:
    _ui->apartmentTableView->setModel(_apartmentModel);
    _ui->apartmentTableView->setItemDelegate(new QSqlRelationalDelegate());
    _ui->apartmentTableView->resizeColumnsToContents();
    _ui->apartmentTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    _ui->apartmentTableView->setEditTriggers(QAbstractItemView::NoEditTriggers); // a táblaszerkesztés kikapcsolása a megjelenítőn
    _ui->apartmentTableView->setColumnHidden(0, true);
    _ui->apartmentTableView->setColumnHidden(6, true);
    _ui->apartmentTableView->setColumnHidden(7, true);
    _ui->apartmentTableView->setColumnHidden(8, true);
    _ui->apartmentTableView->setColumnHidden(10, true);

    connect(_ui->filterButton, SIGNAL(clicked()), this, SLOT(filterButton_Clicked()));

    // táblakijelölés változásának kezelése:
    connect(_buildingSelectionModel, SIGNAL(currentRowChanged(QModelIndex,QModelIndex)), this, SLOT(buildingTableView_SelectionChanged(QModelIndex)));

    setWindowTitle(trUtf8("Épületek szűrése"));
    setFixedSize(600, 590);
}
